@@ -1,3 +1,3 @@
 pub mod as_ref;
 pub mod clone;
-pub mod debug;
+pub mod fmt;
